// jshint maxerr:1000
/*
 * Copyright (c) 2023, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* ===============================================================================================================
   ===============================================================================================================
                          BASIC TAB TOOL 
   ===============================================================================================================
   =============================================================================================================== */

/* ====================== USER INPUT ==========================================================================
 * Take user input and the tool's state and pass it to mmWaveBasic (def in basicToolBackend.js) for processing
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 * Special-Case Handling: tied to a change event in a specific HTML element
 *      - Basic_onReset()      
 *          - Description:  reset the inputs to a default 
 *          - Trigger:      press of RESET button (tied to HTML element in index.gui)
 * 
 *      - Basic_onDesiredCfg()
 *          - Description:  populate inputs with a desired configuration default based on dropdown selection
 *          - Trigger:      change of DESIRED CONFIG dropdown (tied to HTML element in index.gui)
 *
 *      - Basic_onDevice()
 *          - Description:  populate inputs with the selected device's desired config
 *          - Trigger:      change of DEVICE dropdown (tied to HTML element in index.gui)
 * 
 *      - Basic_onSaveConfig()
 *          - Description:  save a usable .cfg file to the user's computer
 *          - Trigger:      press of SAVE CONFIG button (tied to HTML element in index.gui)
 * 
 *      - Basic_onSendToAdv()
 *          - Description:  send the CLI output to the Advanced Tab Tool for more tuning 
 *          - Trigger:      press of SEND TO ADVANCED... button (tied to HTML element in index.gui)
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 * General-Case Handling: catch and hand-off input into any other input element in the tool (mostly textboxes)
 *      - Basic_onInput(mode)      
 *          - Description:  take input from input cells and pass it to the backend for processing. 
 *                          mode = 'init' used for first time init on loading tool
 *          - Trigger:      input into any non-special-case input
 * ===========================================================================================================
 */
var Basic_onDesiredCfg = function() {
    var widget = templateObj.$;
    var changes = {};

    // Get the currently selected desired config and device
    var desired = templateObj.$.ti_widget_droplist_desired_cfg.selectedValue;
    var device = templateObj.$.ti_widget_droplist_device.selectedValue;
    var curr_evm = templateObj.$.ti_widget_droplist_evm.selectedValue;
    var curr_freq_range = templateObj.$.ti_widget_droplist_frequency_range.selectedValue;
    
    // Get a valid evm and frequency range for the device
    var evm_list    = mmWaveBasic.get_evm_list(device);
    var freq_range_list  = mmWaveBasic.get_allowed_freq_range(device);
    Util.Html.updateDroplistOptions(widget.ti_widget_droplist_evm,evm_list);
    Util.Html.updateDroplistOptions(widget.ti_widget_droplist_frequency_range, freq_range_list);
    
    // If the current evm is not valid for the device, select the first valid evm
    var evm;
    if (evm_list.indexOf(curr_evm) === -1) {
        evm = evm_list[0];
    } else {
        evm = curr_evm;
    }

    // If the current frequency range is not valid for the device, select the first valid frequency range
    var freq_range;
    if (freq_range_list.indexOf(curr_freq_range) === -1) {
        freq_range = freq_range_list[0];
    } else {
        freq_range = curr_freq_range;
    }
    
    Util.Html.updateDroplist(widget.ti_widget_droplist_evm, evm);
    Util.Html.updateDroplist(widget.ti_widget_droplist_frequency_range, freq_range);
    
    changes['device'] = device;
    changes['desired_cfg'] = desired;
    changes['evm'] = evm;
    changes['frequency_range'] = freq_range;

    if (desired === 'Short Range Default'){
        Basic_Globals.Defaults.forEach(mapping => {
            changes[mapping[1]] = mapping[2][device]['short'];
            if (mapping[0] === 'ti_widget_droplist_typical_detected_object') {
                Util.Html.updateDroplist(widget[mapping[0]],mapping[2][device]['short']);
                widget.ti_widget_textbox_typical_detected_object.disabled = true;
            } else {
                Util.Html.updateTextbox(widget[mapping[0]],mapping[2][device]['short']);
            }
        });
    }
    else if (desired === 'Long Range Default'){
        Basic_Globals.Defaults.forEach(mapping => {
            changes[mapping[1]] = mapping[2][device]['long'];
            if (mapping[0] === 'ti_widget_droplist_typical_detected_object') {
                Util.Html.updateDroplist(widget[mapping[0]],mapping[2][device]['long']);
                widget.ti_widget_textbox_typical_detected_object.disabled = true;
            } else {
                Util.Html.updateTextbox(widget[mapping[0]],mapping[2][device]['long']);
            }
        });
    }
    mmWaveBasic.updateInput(changes);
    Basic_updateOutput();
    saveObjCookie();
};
var Basic_onDevice = function() {
    var widget = templateObj.$;
    var device = templateObj.$.ti_widget_droplist_device.selectedValue;
    if(device == 'AWR1243' || device == 'AWR2243') {
        widget.ti_widget_button_load_cfg_cfg.disabled = true;
        widget.ti_widget_button_save_config_cfg.disabled = true;
    } else {
        widget.ti_widget_button_load_cfg_cfg.disabled = false;
        widget.ti_widget_button_save_config_cfg.disabled = false;
    }
    Basic_onDesiredCfg();
};
var Basic_onReset = function() {
    var widget = templateObj.$;
    changes = {};

    changes['device']           = 'xWR6843';
    changes['evm']              = 'xWR6843AOP';
    changes['frequency_range']  = '60-64';
    changes['desired_cfg']      = 'Short Range Default';

    var evm_list    = mmWaveBasic.get_evm_list('xWR6843');
    var freq_range  = mmWaveBasic.get_allowed_freq_range('xWR6843');
    Util.Html.updateDroplistOptions(widget.ti_widget_droplist_evm,             evm_list);
    Util.Html.updateDroplistOptions(widget.ti_widget_droplist_frequency_range, freq_range);

    Util.Html.updateDroplist(widget.ti_widget_droplist_device,                   'xWR6843');
    Util.Html.updateDroplist(widget.ti_widget_droplist_evm,                      'xWR6843AOP');
    Util.Html.updateDroplist(widget.ti_widget_droplist_frequency_range,          '60-64');
    Util.Html.updateDroplist(widget.ti_widget_droplist_desired_cfg,           'Short Range Default');
    Basic_InputDefaults.forEach(mapping => {
        changes[mapping[1]] = mapping[2];
            if (mapping[0] === 'ti_widget_droplist_typical_detected_object') {
                Util.Html.updateDroplist(widget[mapping[0]],mapping[2]);
            } else {
                Util.Html.updateTextbox(widget[mapping[0]],mapping[2]);
            }
    });

    mmWaveBasic.updateInput(changes);
    Basic_updateOutput();
    saveObjCookie();
};
var Basic_onSaveConfig = function() {
    var device = templateObj.$.ti_widget_droplist_device.selectedValue;
    if (Basic_Globals.CmdLines.length > 0) {
        // Grab the correct OOB Config Lines
        var fileLines = Basic_Globals.Configs[mmWaveBasic.Input.device][mmWaveBasic.Input.evm];
        var cliLines = Basic_Globals.CmdLines.join('\n').split('\n');

        if (cliLines[0] === '========== ERRORS ==========') {
            alert('Eliminate the errors before attempting to save the configuration to a file.');
            return;
        }

        if (fileLines && device != 'xWRL6432' && device != 'xWRL1432'){
            // Run the same cmd injection algorithm as the adv Save Config:
            var k = 0;
            var j = 0;
            // Generate the file
            fileLines.forEach((line, index, self) => {
                var elements = line.split(/\s+/);
                if (elements[0] === 'profileCfg') {
                    cliLines.forEach(cmd => {
                        // Inject our new cmd into the cfg file
                        if (cmd.includes('profileCfg')) {
                            self[index] = cmd;
                        }
                    });
                }
                else if (elements[0] === 'channelCfg') {
                    cliLines.forEach(cmd => {
                        // Inject our new cmd into the cfg file
                        if (cmd.includes('channelCfg')) {
                            self[index] = cmd;
                        }
                    });
                }
                else if (elements[0] === 'adcbufCfg') {
                    cliLines.forEach(cmd => {
                        // Inject our new cmd into the cfg file
                        if (cmd.includes('adcbufCfg')) {
                            self[index] = cmd;
                        }
                    });
                }
                else if (elements[0] === 'chirpCfg') {
                    j = 0;
                    cliLines.forEach(cmd => {
                        // Inject our new cmd into the cfg file
                        if (cmd.includes('chirpCfg')) {
                            if (j === k) {
                                self[index] = cmd;
                            }
                            j++;
                        }
                    });
                    k++;
                }
                else if (elements[0] === 'frameCfg') {
                    cliLines.forEach(cmd => {
                        // Inject our new cmd into the cfg file
                        if (cmd.includes('frameCfg')) {
                            self[index] = cmd;
                        }
                    });
                }
            });
            // Join the modified lines
            var output = fileLines.join('\n') + '\n\n';
            // output the file
            gc.File.saveBrowserFile(output, { filename: Util.FileIO.getUniqueFileName('profile','.cfg') }, function (e1) {});
        }
        else if(fileLines && device == 'xWRL6432' || device == 'xWRL1432'){
            // Run the same cmd injection algorithm as the adv Save Config:
            var k = 0;
            var j = 0;
            // Generate the file
            fileLines.forEach((line, index, self) => {
                var elements = line.split(/\s+/);
                if (elements[0] === 'channelCfg') {
                    cliLines.forEach(cmd => {
                        // Inject our new cmd into the cfg file
                        if (cmd.includes('channelCfg')) {
                            self[index] = cmd;
                        }
                    });
                }
                else if (elements[0] === 'chirpComnCfg') {
                    cliLines.forEach(cmd => {
                        // Inject our new cmd into the cfg file
                        if (cmd.includes('chirpComnCfg')) {
                            self[index] = cmd;
                        }
                    });
                }
                else if (elements[0] === 'chirpTimingCfg') {
                    cliLines.forEach(cmd => {
                        // Inject our new cmd into the cfg file
                        if (cmd.includes('chirpTimingCfg')) {
                            self[index] = cmd;
                        }
                    });
                }
                else if (elements[0] === 'frameCfg') {
                    cliLines.forEach(cmd => {
                        // Inject our new cmd into the cfg file
                        if (cmd.includes('frameCfg')) {
                            self[index] = cmd;
                        }
                    });
                }
            });
            // Join the modified lines
            var output = fileLines.join('\n') + '\n\n';
            // output the file
            gc.File.saveBrowserFile(output, { filename: Util.FileIO.getUniqueFileName('profile','.cfg') }, function (e1) {});
        }
        else {
            alert('Make sure to select a device and evm before trying to save a CFG');
            return;
        }
    } 
    else {
        alert('SAVE CONFIG only works when no errors are present on the basic tab.\n\nResolve errors before saving config!');
    }

};
var Basic_onInput = function(mode) {
    // For ease of reading, get the widget context
    var widget = templateObj.$;
    if (mode === 'init') {
        Basic_Globals.Inputs.forEach(widgetMap => {
            // Initially populate all the input widgets
            if (widgetMap[2] === 'dropdown') {
                Util.Html.updateDroplist(widget[widgetMap[0]],mmWaveBasic.Input[widgetMap[1]]);
                // Handle the other dropdowns conditionally on the device value
                if (widgetMap[0] === 'ti_widget_droplist_device') {
                    var evm_list    = mmWaveBasic.get_evm_list(mmWaveBasic.Input.device);
                    var freq_range  = mmWaveBasic.get_allowed_freq_range(mmWaveBasic.Input.device);
                    Util.Html.updateDroplistOptions(widget.ti_widget_droplist_evm,             evm_list);
                    Util.Html.updateDroplistOptions(widget.ti_widget_droplist_frequency_range, freq_range);
                    // If the previous selections in num_rx, num_tx, freq_range are no longer valid, update
                    if (evm_list.indexOf(widget.ti_widget_droplist_evm.selectedValue) < 0){
                        Util.Html.updateDroplist(widget.ti_widget_droplist_evm, evm_list[evm_list.length-1]);
                    }
                    if (freq_range.indexOf(widget.ti_widget_droplist_frequency_range.selectedValue) < 0){
                        Util.Html.updateDroplist(widget.ti_widget_droplist_frequency_range, freq_range[freq_range.length-1]);
                    }
                }
                // Handle the textbox conditionally on the detectObj dropdown
                if (widgetMap[0] === 'ti_widget_droplist_typical_detected_object') {
                    if (mmWaveBasic.Input.typical_detected_object !== 'Custom') {
                        var obj_rcs = mmWaveBasic.get_typical_detected_object_to_rcs(mmWaveBasic.Input.typical_detected_object);
                        mmWaveBasic.Input.typical_detected_val = obj_rcs;
                        Util.Html.updateTextbox(templateObj.$.ti_widget_textbox_typical_detected_object, obj_rcs);
                        widget.ti_widget_textbox_typical_detected_object.disabled = true;
                    } else {
                        widget.ti_widget_textbox_typical_detected_object.disabled = false;
                    }
                }
            } else {
                Util.Html.updateTextbox(widget[widgetMap[0]],mmWaveBasic.Input[widgetMap[1]]);
            }
        });
        
        // Pass the changes to the backend to calculate outputs
        mmWaveBasic.updateInput();
    } else {
        // Object to store all changes to the input fields, passed to mmWaveBasic.updateInput(...)
        var changes = {}; 

        // Loop through every input field and grab its current value from the front-end
        Basic_Globals.Inputs.forEach(widgetMap => {
            // int type input fields
            if (widgetMap[2] == 'int') {
                var new_val = parseInt(widget[widgetMap[0]].value);
                if (isNaN(new_val) === false){
                    changes[widgetMap[1]] = new_val;
                    Util.Html.updateTextbox(widget[widgetMap[0]], new_val);
                }
            }
            // float type input fields
            else if (widgetMap[2] == 'float') {
                var new_val = parseFloat(widget[widgetMap[0]].value);
                if (isNaN(new_val) === false){
                    changes[widgetMap[1]] = new_val;
                    Util.Html.updateTextbox(widget[widgetMap[0]], new_val);
                }
            }
            // dropdown type input fields
            else {
                var new_val = widget[widgetMap[0]].selectedValue;
                changes[widgetMap[1]] = new_val;
                // if typical det obj dropdown, change the custom textbox
                if (widgetMap[0] === 'ti_widget_droplist_typical_detected_object') {
                    if (new_val !== 'Custom') {
                        var obj_rcs = mmWaveBasic.get_typical_detected_object_to_rcs(new_val);
                        Util.Html.updateTextbox(templateObj.$.ti_widget_textbox_typical_detected_object, obj_rcs);
                        widget.ti_widget_textbox_typical_detected_object.disabled = true;
                    } else {
                        widget.ti_widget_textbox_typical_detected_object.disabled = false;
                    }
                }
            }
        });

        // Pass the changes to the backend
        mmWaveBasic.updateInput(changes);
    }
    // Update the output fields to reflect the changes made in the input fields
    Basic_updateOutput();
};
var Basic_onSendToAdv = function() {
    // Populate the auto-populate tool
    if (Basic_Globals.CmdLines.length > 0) {
        Adv_onAutoPopulate(Basic_Globals.CmdLines, mmWaveBasic.Input.device);
        
        // reflect the device textboxs
        var changes = {};
        changes['device'] = mmWaveBasic.Input.device; 
        changes['evm'] = mmWaveBasic.Input.evm;
        var evmList     = mmWaveAdvanced.get_allowed_evms(mmWaveBasic.Input.device);
        var adcModeList = mmWaveAdvanced.get_allowed_adcMode(mmWaveBasic.Input.device);
        var sdkVersion  = mmWaveAdvanced.get_supported_SDK(mmWaveBasic.Input.device); // sdkVersion = 0 is SDK 3, sdkVersion = 1 is SDK 5
        Util.Html.updateDroplistOptions(templateObj.$.ti_widget_dropdown_cc_evm,     evmList);
        Util.Html.updateDroplistOptions(templateObj.$.ti_widget_dropdown_cc_adcMode, adcModeList);
        Util.Html.updateDroplist(templateObj.$.ti_widget_dropdown_cc_device, mmWaveBasic.Input.device);
        Util.Html.updateDroplist(templateObj.$.ti_widget_dropdown_cc_evm, mmWaveBasic.Input.evm);
        if (adcModeList.indexOf(templateObj.$.ti_widget_dropdown_cc_adcMode.selectedValue) < 0){
            Util.Html.updateDroplist(templateObj.$.ti_widget_dropdown_cc_adcMode, adcModeList[adcModeList.length-1]);
            changes['adcMode'] = adcModeList[adcModeList.length-1];
        }

        /*update tab to show correct version for SDK here using sdkVersion*/
        Util.Html.selectTab(templateObj.$.ti_widget_tabcontainer_sdk, sdkVersion);
        // Pass the changes to the backend
        mmWaveAdvanced.updateInput(changes);
        // Update the output fields to reflect the changes made in the input fields
        Adv_updateOutput();
        // Notify the user
        templateObj.$.ti_widget_textbox_cfg_console.value = '========== Successfully sent to ADVANCED ==========\n\n' + templateObj.$.ti_widget_textbox_cfg_console.value;
        // Switch tab view
        Util.Html.selectTab(templateObj.$.tool_tabcontainer, 1); // 0 is basic, 1 is advanced, 2 is power
        templateObj.$.viewport.scrollTop = 0;
    } else {
        alert('SEND TO ADVANCED only works when no errors are present on the basic tab.\n\nResolve errors before sending the commands to the ADVANCED tool!');
    }
};
/* ====================== OUTPUT ==========================================================================
 * Take the processed output data from mmWaveBasic (def in basicToolBackend.js) and update the output fields
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *      - Basic_updateOutput()      
 *          - Description:  populate the output fields with the processed results in mmWaveBasic.Output
 *          - Trigger:      call after Basic_onInput() and mmWaveBasic.updateInput() 
 * 
 *      - Basic_updateErrors()
 *          - Description:  based on the tool's state, determined and apply error-checks, create feedback list
 *          - Trigger:      called by Basic_updateOutput()
 *
 *      - Basic_generateCliOutput(feedbackStr)
 *          - Description:  take the feedback list from Basic_updateErrors() and create terminal output
 *          - Trigger:      called by Basic_updateOutput() after Basic_updateErrors()
 * ===========================================================================================================
 */
var Basic_updateOutput = function() {
    // For ease of reading, get the widget context
    var widget = templateObj.$;

    // Reflect the updated output values computed in the mmWaveBasic backend to the GUI frontend
    Basic_Globals.Outputs.forEach(widgetMap => {
        if (widgetMap[2] === 'float'){
            Util.Html.updateTextbox(widget[widgetMap[0]], mmWaveBasic.Output[widgetMap[1]], 3);  // Limit to 3 decimal places
        }
        else{
            Util.Html.updateTextbox(widget[widgetMap[0]], mmWaveBasic.Output[widgetMap[1]]);
        }
    });

    // Update the conditional error checking
    var feedback = Basic_updateErrors();
    Basic_generateCliOutput(feedback);
};
var Basic_updateErrors = function() {
    // For ease of reading, get the widget context
    var widget = templateObj.$;

    // Strings that hold all the feedback from the error/warning checks
    var errString = '';
    var warnString = '';

    // Clear errors and error related...
    Basic_Globals.Inputs.forEach(widgetMap => {
        // Get the parent container from the widget name:   'ti_widget_textbox_num_rx' -> 'ti_widget_container_num_rx'
        var parentContainer = widgetMap[0].replace('textbox','container');
        Util.Html.highlightPropWithPrecedence(widget[parentContainer],0,'error1');
    });
    Basic_Globals.Outputs.forEach(widgetMap => {
        var parentContainer = widgetMap[0].replace('textbox','container');
        Util.Html.highlightPropWithPrecedence(widget[parentContainer],0,'error1');
    });
    Basic_ErrorMaps.Related.forEach(cellName => Util.Html.highlightPropWithPrecedence(widget[cellName],0,'conditional'));
    Basic_ErrorMaps.Warn.forEach(cellName => Util.Html.highlightPropWithPrecedence(widget[cellName],0,'warning'));

    // Evaluate the error rules and apply conditional formatting as necessary
    Basic_ErrorMaps.Err.forEach(map => {
        var widgetNames, isErr, errText;
        widgetNames  = map[0];              // grab the widgets involved in this error check
        [isErr, errText] = map[1]();        // grab the return value from the error check function call
        if (errText !== '') errString += errText + '\n\n'; 
        widgetNames.forEach(widgetName => Util.Html.highlightPropWithPrecedence(widget[widgetName],isErr,'error1'));
    });

    // Evaluate the warning rules and apply formatting as necessary
    Basic_ErrorMaps.Warn.forEach(map => {
        var widgetNames, isWarn, warnText;
        widgetNames  = map[0];              // grab the widgets involved in this error check
        [isWarn, warnText] = map[1]();        // grab the return value from the error check function call
        if (warnText !== '') warnString += warnText + '\n\n'; 
        widgetNames.forEach(widgetName => Util.Html.highlightPropWithPrecedence(widget[widgetName],isWarn,'warning'));
    });

    // Check generally for any negative or NaN cells in the input and output side and highlight them
    Basic_Globals.Inputs.forEach(widgetMap => {
        var widgetName = widgetMap[0];
        var widgetVal  = mmWaveBasic.Input[widgetMap[1]];
        // Do not perform the check on the dropdowns:
        if (!widgetName.includes('droplist')){
            // Get the parent container from the widget name:   'ti_widget_textbox_num_rx' -> 'ti_widget_container_num_rx'
            var parentContainer = widgetName.replace('textbox','container');
            if (widgetVal < 0 || isNaN(widgetVal)) {
                Util.Html.highlightPropWithPrecedence(widget[parentContainer],1,'error1');
                warnString += widgetMap[1] + ' is negative or NaN. This could be an artifact of calculation, but most likely represents an invalid input. Please check your inputs.\n\n';
            }
        }
    });
    Basic_Globals.Outputs.forEach(widgetMap => {
        var widgetName = widgetMap[0];
        var widgetVal  = mmWaveBasic.Output[widgetMap[1]];
        // Do not perform the check on the dropdowns:
        if (!widgetName.includes('droplist')){
            // Get the parent container from the widget name:   'ti_widget_textbox_num_rx' -> 'ti_widget_container_num_rx'
            var parentContainer = widgetName.replace('textbox','container');
            if (widgetVal < 0 || isNaN(widgetVal)) {
                Util.Html.highlightPropWithPrecedence(widget[parentContainer],1,'error1');
                warnString += widgetMap[1] + ' is negative or NaN. This could be an artifact of calculation, but most likely represents an invalid input. Please check your inputs.\n\n';
            }
        }
    });

    return [errString,warnString];
};
var Basic_generateCliOutput = function(feedbackStr) {
    //var Input = mmWaveAdvanced.Input;
    var Input = mmWaveBasic.Input;
    var output = '';
    var errString = feedbackStr[0];
    var warnString = feedbackStr[1];
    if (Input.device == 'AWR1243' || Input.device == 'AWR2243'){
        output = output +
        '% ***************************************************************\n' +
        [' FRONT END DEVICE ONLY, DOES NOT REQUIRE CONFIGURATION FILE:', Input.device].join(' ') +
        '\n% ***************************************************************\n';
    }
    if(errString === '' && Input.device != 'AWR1243' && Input.device != 'AWR2243'){
        //sdk 3 devices only
            // Create your cliCmds to output
            if (Input.device                     !== '' &&
                Input.evm                        !== '' &&
                Input.frequency_range            !== '' &&
                Input.ambient_temperature_degC   !== '' &&
                Input.maximum_detectable_range   !== '' &&
                Input.range_resolution           !== '' &&
                Input.maximum_velocity_kmph      !== '' &&
                Input.velocity_resolution_kmph   !== '' &&
                Input.measurement_rate           !== '' &&
                Input.typical_detected_object    !== '' &&
                Input.typical_detected_val       !== '' &&
                Input.detection_loss             !== '' &&
                Input.system_loss                !== '' &&
                Input.implementation_margin      !== '' &&
                Input.detection_SNR              !== '' )
            {
                if(Input.device == 'xWRL6432' || Input.device == 'xWRL1432'){
                    var channelCfg5, chirpComnCfg, chirpTimingCfg, framecfg5 = '';
                    channelCfg5 = 'channelCfg 7 3 0';
                    chirpComnCfg = 'chirpComnCfg ' + Util.Format.toPrecision(100/mmWaveBasic.Output.sampling_frequency, 0) + ' 0' + ' 0 '+ mmWaveBasic.Output.number_of_samples_per_chirp +' 4 '+ Util.Format.toPrecision(mmWaveBasic.Output.ramp_end_time, 0) +' 0';
                    chirpTimingCfg = 'chirpTimingCfg '+ mmWaveBasic.Output.idle_time_minimum + ' ' + (Util.Format.toPrecision((mmWaveBasic.Output.sampling_frequency)*mmWaveBasic.Output.adc_valid_start_time_2, 0)) + ' 0 ' + Util.Format.toPrecision(mmWaveBasic.Output.ramp_slope, 2) + ' ' + mmWaveBasic.Output.starting_frequency;
                    framecfg5 = 'frameCfg ' + Util.Format.toPrecision(mmWaveBasic.Output.chirps_in_burst) + ' 0 ' + (Util.Format.toPrecision(mmWaveBasic.Output.burst_periodicity)) + ' ' + mmWaveBasic.Output.bursts_in_frame + ' ' + Util.Format.toPrecision(1 / Input.measurement_rate * 1000, 2) + ' 0';
                    factoryCalibCfg = 'factoryCalibCfg 1 0 30 0 0x1ff000' 
                    // Insert the valid cliCmds into the global storing it
                    Basic_Globals.CmdLines = [channelCfg5,chirpComnCfg, chirpTimingCfg, framecfg5, factoryCalibCfg];
                }
                else{
                    var profileCfg, channelCfg3, adcbufCfg, chirpCfg1, chirpCfg2, chirpCfg3, frameCfg3 = '';
                    profileCfg = 'profileCfg 0 ' + mmWaveBasic.Output.starting_frequency + ' ' + mmWaveBasic.Output.idle_time_minimum + ' ' + mmWaveBasic.Output.adc_valid_start_time_2 + ' ' + Util.Format.toPrecision(mmWaveBasic.Output.ramp_end_time, 2) + ' 0 0 ' + Util.Format.toPrecision(mmWaveBasic.Output.ramp_slope, 2) + ' 1 ' + mmWaveBasic.Output.number_of_samples_per_chirp + ' ' + Util.Format.toPrecision(mmWaveBasic.Output.sampling_frequency * 1000, 0) + ' 0 0 30'
                    adcbufCfg = 'adcbufCfg -1 0 1 1 1';
                    if (Input.device === 'xWR1642') {
                        channelCfg3 = 'channelCfg 15 3 0';
                        chirpCfg1 = 'chirpCfg 0 0 0 0 0 0 0 1';
                        chirpCfg2 = 'chirpCfg 1 1 0 0 0 0 0 2';
                        frameCfg3 = 'frameCfg 0 1 ' + mmWaveBasic.Output.min_num_of_chirp_loops + ' 0 ' + Util.Format.toPrecision(1 / mmWaveBasic.Input.measurement_rate * 1000, 2) + ' 1 0';
                        // Insert the valid cliCmds into the global storing it
                        Basic_Globals.CmdLines = [profileCfg, channelCfg3, adcbufCfg, chirpCfg1, chirpCfg2, frameCfg3];
                    }
                    else {
                        channelCfg3 = 'channelCfg 15 7 0';
                        chirpCfg1 = 'chirpCfg 0 0 0 0 0 0 0 1';
                        chirpCfg2 = 'chirpCfg 1 1 0 0 0 0 0 2';
                        chirpCfg3 = 'chirpCfg 2 2 0 0 0 0 0 4';
                        frameCfg3 = 'frameCfg 0 2 ' + mmWaveBasic.Output.min_num_of_chirp_loops + ' 0 ' + Util.Format.toPrecision(1 / mmWaveBasic.Input.measurement_rate * 1000, 2) + ' 1 0';
                        // Insert the valid cliCmds into the global storing it
                        Basic_Globals.CmdLines = [profileCfg, channelCfg3, adcbufCfg, chirpCfg1, chirpCfg2, chirpCfg3, frameCfg3];
                    }
                }

                // Create your output value
                output = output + 
                '% ===============================================================================\n' +
                '% NOTE: Profile config and frame config provided below are not guaranteed to work\n' + 
                '% with mmWaveBasic SDK without calibrating digital processing chain.\n' +
                '% ===============================================================================\n' +
                Basic_Globals.CmdLines.join('\n');
            } else {
                Basic_Globals.CmdLines = [];
                output = output + 'Make sure to provide values for all input fields.\n'
            }
        // Append warnings
        if (warnString !== '') output += '\n\n========== WARNINGS ==========\n' + warnString;
        // Send the output
        templateObj.$.ti_widget_textbox_cfg_console.value = output;
    } 

    else {
        Basic_Globals.CmdLines = [];
        if(errString !== ''){
            output = output + '========== ERRORS ==========\n' + errString;
        }
        templateObj.$.ti_widget_textbox_cfg_console.value = output;
        // Append warnings
        if (warnString !== '') templateObj.$.ti_widget_textbox_cfg_console.value += '\n\n========== WARNINGS ==========\n' + warnString;
    }
};